code	segment	'code'
	assume	cs:code,ds:code

newsp	equ	buff+256*16+512-2	; stack location
mempg	equ	newsp/16+1		; required paras of memory

	org	100h			; COM entry point

; ----- main

start:	mov	ax,1a00h		; got VGA?
	int	10h
	cmp	al,1ah
	je	isvga
	mov	dx,offset vmsg		; show message and abort
	mov	ah,09h
	int	21h
	jmp	short exit

isvga:	cli				; move stack pointer
	mov	sp,newsp
	sti

	mov	bx,mempg		; resize memory block to required
	mov	ah,4ah
	int	21h

	mov	ax,3510h		; get current INT 10 handler
	int	21h
	mov	word ptr old10,bx
	mov	word ptr old10+2,es
	push	cs
	pop	es

	mov	dx,offset new10		; install new INT 10 handler
	mov	ax,2510h
	int	21h

	mov	word ptr tail+2,cs	; set segs in param block
	mov	word ptr fcb1+2,cs
	mov	word ptr fcb2+2,cs

	mov	dx,offset prog		; EXEC child process
	mov	bx,offset parm
	mov	ax,4b00h
	int	21h

	mov	dx,word ptr old10	; restore previous INT 10 handler
	mov	ds,word ptr old10+2
	mov	ax,2510h
	int	21h
	push	cs
	pop	ds

exit:	mov	ax,4c00h		; exit to DOS
	int	21h

; ----- INT 10 handler

new10:	cmp	ax,0003h		; setting video mode 3?
	je	trap

char:	cmp	ax,1100h		; loading custom font?
	je	trap

	jmp	cs:old10		; forward anything unhandled

trap:	push	ax			; save registers
	push	bx
	push	cx
	push	dx
	push	es
	push	ds
	push	si
	push	di
	push	bp

	mov	di,offset buff		; conversion buffer destination

	cmp	ax,0003h		; mode 3 loads 8x8 font instead
	je	mode

	push	cx			; convert and load custom font
	call	double
	pop	cx
	jmp	short load

mode:	mov	bh,3			; convert first half of 8x8 chars
	call	half
	mov	bh,4			; convert second half of 8x8 chars
	call	half
	xor	bl,bl			; params for loading full set
	mov	cx,256
	xor	dx,dx

load:	mov	ax,1100h		; load the converted 8x16 chars
	mov	bh,16
	mov	bp,offset buff
	pushf
	call	cs:old10

	pop	bp			; restore registers
	pop	di
	pop	si
	pop	ds
	pop	es
	pop	dx
	pop	cx
	pop	bx
	pop	ax

	iret				; return from interrupt

; ----- subroutines

half:	mov	ax,1130h		; font information function
	pushf
	call	cs:old10

	mov	cx,128			; convert half set of 8x8 chars
	call	double

	ret

double:	push	es			; arrange source and destination
	pop	ds
	push	cs
	pop	es
	mov	si,bp

	shl	cx,1			; char count x 8 lines
	shl	cx,1
	shl	cx,1

	cld

dloop:	lodsb				; read a line from source
	mov	ah,al
	stosw				; write it twice to destination
	loop	dloop

	ret

; ----- data

old10	dd	0			; INT 10 handler address

parm	dw	0			; default env seg
tail	dd	80h			; command line tail
fcb1	dd	5ch			; first FCB
fcb2	dd	6ch			; second FCB

prog	db	'INDENTUR.COM',0	; program filename
vmsg	db	'VGA required$'		; error message

buff	equ	$			; font conversion buffer

code	ends

	end	start
